declare function isNullOrUndefined(value: any): value is undefined | null;
/**
 * Strict object type check. Only returns true
 * for plain JavaScript objects.
 */
declare function isPlainObject(value: any): value is Record<string, any>;
declare function isNumber(value: any): value is number;
declare function isString(value: any): value is string;
declare function isUndefined(value: any): boolean;
declare function isPromise(promise: any): boolean;
/**
 * Check if the input string contains only numeric characters.
 */
declare function isNumeric(value: string): boolean;
declare function isFunction(fn: unknown): fn is (...args: any[]) => any;
/**
 * Returns true if the object is empty. Empty is defined as:
 * - null
 * - undefined
 * - a string with zero length
 * - an array with no elements
 * - a collection with no elements
 */
declare function isEmpty(value: any): value is null | undefined | 0;
declare function isEqual(a: any, b: any): boolean;

/**
 * Merge objects or arrays
 */
declare function merge(target: any, ...sources: any[]): any;
/**
 * Find the first found value in objects or arrays by its path
 */
declare function get(path: string | string[], ...args: any[]): any;
/**
 * Returns the index of the first element in the array that satisfies the provided testing function.
 */
declare function findIndex(arr: any[], fn: (...args: any[]) => boolean): number;
/**
 * Creates a new array with all sub-array elements concatenated into it recursively up to the specified depth.
 */
declare function flatArray(arr: any[], deep?: number): any[];

declare type Data = Record<string, any>;
declare function interpolate(format: string, data?: Data | Data[], interpolator?: RegExp): string;

export { findIndex, flatArray, get, interpolate, isEmpty, isEqual, isFunction, isNullOrUndefined, isNumber, isNumeric, isPlainObject, isPromise, isString, isUndefined, merge };
