const _toString = Object.prototype.toString;
export function isNullOrUndefined(value) {
  return value === null || value === undefined;
}
/**
 * Strict object type check. Only returns true
 * for plain JavaScript objects.
 */

export function isPlainObject(value) {
  return _toString.call(value) === '[object Object]';
}
export function isNumber(value) {
  return typeof value === 'number' && !isNaN(value);
}
export function isString(value) {
  return typeof value === 'string';
}
export function isUndefined(value) {
  return value === undefined;
}
export function isPromise(promise) {
  return promise && typeof promise.then === 'function' && _toString.call(promise) === '[object Promise]';
}
/**
 * Check if the input string contains only numeric characters.
 */

export function isNumeric(value) {
  const val = String(value);
  return val.length > 0 && !isNaN(+val);
}
export function isFunction(fn) {
  return typeof fn === 'function';
}
/**
 * Returns true if the object is empty. Empty is defined as:
 * - null
 * - undefined
 * - a string with zero length
 * - an array with no elements
 * - a collection with no elements
 */

export function isEmpty(value) {
  if (isNullOrUndefined(value)) {
    return true;
  }

  let length;

  if (Array.isArray(value) || isString(value)) {
    length = value.length;
  }

  if (isPlainObject(value)) {
    length = Object.keys(value).length;
  }

  return length === 0;
}
export function isEqual(a, b) {
  if (a === b) return true;

  if (a instanceof Date && b instanceof Date && a.getTime() !== b.getTime()) {
    // If the values are Date, compare them as timestamps
    return false;
  }

  if (a !== Object(a) || b !== Object(b)) {
    // If the values aren't objects, they were already checked for equality
    return false;
  }

  const props = Object.keys(a);

  if (props.length !== Object.keys(b).length) {
    // Different number of props, don't bother to check
    return false;
  }

  return props.every(p => isEqual(a[p], b[p]));
}